package com.flyfish.framework.dict.service;


import com.flyfish.framework.dict.domain.Dictionary;
import com.flyfish.framework.dict.repository.DictionaryRepository;
import com.flyfish.framework.service.impl.BaseReactiveServiceImpl;
import com.flyfish.framework.utils.RedisOperations;
import org.springframework.data.mongodb.core.MongoOperations;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Mono;

import javax.annotation.Resource;
import java.util.Optional;

/**
 * 字典表服务
 *
 * @author wangyu
 */
@Service
public class DictionaryService extends BaseReactiveServiceImpl<Dictionary> {

    private final String DICT_KEY = "dict-";

    @Resource
    private MongoOperations mongoOperations;
    @Resource
    private RedisOperations redisOperations;

    /**
     * 通过code查询
     *
     * @return 结果
     */
    public Mono<Dictionary> getByCode(String code) {
        DictionaryRepository repository = this.getRepository();
        return repository.findByCode(code);
    }

    /**
     * 通过code同步查询
     *
     * @param code 编码
     * @return 结果
     */
    public Optional<Dictionary> get(String code) {
        String cacheKey = DICT_KEY + code;
        if (redisOperations.hasKey(cacheKey)) {
            return Optional.of(redisOperations.get(cacheKey)).map(obj -> (Dictionary) obj);
        } else {
            Dictionary dictionary = mongoOperations.findOne(Query.query(Criteria.where("code").is(code)).limit(1),
                    Dictionary.class);
            if (null != dictionary) {
                redisOperations.set(cacheKey, dictionary);
                return Optional.of(dictionary);
            } else {
                return Optional.empty();
            }
        }
    }
}
